@@ -1,33 +1,27 @@
-#  Copyright (c) 2007, Enthought, Inc.
-#  License: BSD Style.
-
 """
+Edit a List of Strings
+
 Simple demonstration of the ListStrEditor, which can be used for editing and
 displaying lists of strings, or other data that can be logically mapped to a
 list of strings.
 """
 
-#-- Imports --------------------------------------------------------------------
-
-from traits.api \
-    import HasTraits, List, Str
-
-from traitsui.api \
-    import View, Item, ListStrEditor
+from traits.api import HasTraits, List, Str
+from traitsui.api import View, Item, ListStrEditor
 
 #-- ShoppingListDemo Class -----------------------------------------------------
 
-class ShoppingListDemo ( HasTraits ):
+class ShoppingListDemo (HasTraits):
 
     # The list of things to buy at the store:
-    shopping_list = List( Str )
+    shopping_list = List(Str)
 
     #-- Traits View Definitions ------------------------------------------------
 
     view = View(
-        Item( 'shopping_list',
+        Item('shopping_list',
               show_label = False,
-              editor = ListStrEditor( title = 'Shopping List', auto_add = True )
+              editor = ListStrEditor(title = 'Shopping List', auto_add = True)
         ),
         title     = 'Shopping List',
         width     = 0.2,
@@ -37,7 +31,7 @@ class ShoppingListDemo ( HasTraits ):
 
 #-- Set up the Demo ------------------------------------------------------------
 
-demo = ShoppingListDemo( shopping_list = [
+demo = ShoppingListDemo(shopping_list = [
     'Carrots',
     'Potatoes (5 lb. bag)',
     'Cocoa Puffs',
@@ -48,7 +42,7 @@ demo = ShoppingListDemo( shopping_list = [
     'Paper towels',
     'Soup (3 cans)',
     'Laundry detergent'
-] )
+])
 
 # Run the demo (in invoked from the command line):
 if __name__ == '__main__':
