@@ -31,9 +31,9 @@ from configobj import ConfigObj
 from traits.api import (HasTraits, HasPrivateTraits, Str, Instance, Property, Any,
     Code, HTML, true, false, Dict)
 
-from ..api import (TreeEditor, ObjectTreeNode, TreeNodeObject, View, Item,
+from traitsui.api import (TreeEditor, ObjectTreeNode, TreeNodeObject, View, Item,
     VSplit, Tabbed, VGroup, HGroup, Heading, Handler, UIInfo, InstanceEditor,
-    Include, spring)
+    HTMLEditor, Include, spring)
 
 from os import listdir
 
@@ -686,7 +686,8 @@ path_view = View(
         Item( 'description',
               label      = 'Description',
               show_label = False,
-              style      = 'readonly'
+              style      = 'readonly',
+              editor=HTMLEditor(format_text=True)
         ),
         Item( 'source',
               label      = 'Source',
@@ -697,16 +698,17 @@ path_view = View(
         id     = 'tabbed'
     ),
     id      = 'traitsui.demos.demo.path_view',
-    dock    = 'horizontal'
+    #dock    = 'horizontal'
 )
 
 demo_view = View(
-     VSplit(
+     #VSplit(
          Tabbed(
              Item( 'description',
                    label      = 'Description',
                    show_label = False,
-                   style      = 'readonly'
+                   style      = 'readonly',
+                   editor=HTMLEditor(format_text=True)
              ),
              Item( 'source',
                    label      = 'Source',
@@ -723,21 +725,26 @@ demo_view = View(
                    # for tabbed items. Needs more investigation.
                    visible_when = 'demo',
              ),
-             export = 'DockWindowShell',
-             id     = 'tabbed',
-         ),
-         VGroup(
              Item( 'log',
                    show_label = False,
                    style      = 'readonly'
              ),
-             label = 'Log'
+             export = 'DockWindowShell',
+             id     = 'tabbed',
          ),
-         export = 'DockWindowShell',
-         id     = 'vsplit'
-     ),
+         # JDM moving log panel provisionally to its own tab, distracting here.
+         #VGroup(
+             #Item( 'log',
+                   #show_label = False,
+                   #style      = 'readonly'
+             #),
+             #label = 'Log'
+         #),
+         #export = 'DockWindowShell',
+         #id     = 'vsplit'
+     #),
      id      = 'traitsui.demos.demo.file_view',
-     dock    = 'horizontal',
+     #dock    = 'horizontal',
      handler = demo_file_handler
 )
 
@@ -787,10 +794,17 @@ class Demo ( HasPrivateTraits ):
             ),
             title     = self.title,
             id        = 'traitsui.demos.demo.Demo',
-            dock      = 'horizontal',
+            #dock      = 'horizontal',
             resizable = True,
+            # JDM: Seems that QT interface does not deal well with these size
+            # limits.
+            # With them, we get repeated:
+            #   Object::disconnect: Parentheses expected, signal AdvancedCodeWidget::lostFocus
+            # But without them, it throws an exception on exit:
+            #    Internal C++ object (_StickyDialog) already deleted.
+            # No, actually sometimes we get the latter even with them.
             width     = 950,
-            height    = 600
+            height    = 900
         )
         return traits_view
 
