@@ -17,10 +17,13 @@
 #-------------------------------------------------------------------------------
 
 import copy
+import collections
+import logging
 
 from pyface.qt import QtCore, QtGui
 
 from pyface.resource_manager import resource_manager
+from pyface.timer.api import do_later
 from traits.api import Any, Event
 from traits.trait_base import enumerate
 from traitsui.api import TreeNode, ObjectTreeNode, MultiTreeNode
@@ -30,11 +33,15 @@ from traitsui.menu import Menu, Action, Separator
 
 from clipboard import clipboard, PyMimeData
 from editor import Editor
-from helper import open_fbi, pixmap_cache
+from helper import pixmap_cache
+
+logger = logging.getLogger(__name__)
 
 #-------------------------------------------------------------------------------
 #  The core tree node menu actions:
 #-------------------------------------------------------------------------------
+from traitsui.ui_traits import SequenceTypes
+
 
 NewAction    = 'NewAction'
 CopyAction   = Action( name         = 'Copy',
@@ -73,6 +80,10 @@ class SimpleEditor ( Editor ):
     # The currently selected object
     selected = Any
 
+    # The event fired when a tree node is activated by double clicking or
+    # pressing the Enter key on a node.
+    activated = Event
+
     # The event fired when a tree node is clicked on:
     click = Event
 
@@ -82,6 +93,9 @@ class SimpleEditor ( Editor ):
     # The event fired when the application wants to veto an operation:
     veto = Event
 
+    # The vent fired when the application wants to refresh the viewport.
+    refresh = Event
+
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
     #  widget:
@@ -108,8 +122,9 @@ class SimpleEditor ( Editor ):
                     factory._editor = self
 
                     # Create the trait editor panel:
-                    self.control = QtGui.QWidget()
-                    parent.addWidget(self.control)
+                    self.control = sa = QtGui.QScrollArea()
+                    sa.setFrameShape(QtGui.QFrame.NoFrame)
+                    sa.setWidgetResizable(True)
                     self.control._node_ui = self.control._editor_nid = None
 
                     # Check to see if there are any existing editors that are
@@ -151,6 +166,7 @@ class SimpleEditor ( Editor ):
 
                 self._editor = sa = QtGui.QScrollArea()
                 sa.setFrameShape(QtGui.QFrame.NoFrame)
+                sa.setWidgetResizable(True)
                 sa._node_ui = sa._editor_nid = None
 
                 if factory.orientation == 'horizontal':
@@ -174,7 +190,9 @@ class SimpleEditor ( Editor ):
         self._undoable = []
 
         # Synchronize external object traits with the editor:
+        self.sync_value( factory.refresh,  'refresh' )
         self.sync_value( factory.selected, 'selected' )
+        self.sync_value( factory.activated,'activated', 'to' )
         self.sync_value( factory.click,    'click',  'to' )
         self.sync_value( factory.dclick,   'dclick', 'to' )
         self.sync_value( factory.veto,     'veto',   'from' )
@@ -187,9 +205,23 @@ class SimpleEditor ( Editor ):
         """ Handles the **selection** event.
         """
         try:
-            self._tree.setCurrentItem(self._object_info(selection)[2])
+            tree = self._tree
+            if (not isinstance(selection, basestring) and
+                isinstance(selection, collections.Iterable)):
+
+                item_selection = QtGui.QItemSelection()
+                for sel in selection:
+                    item = self._object_info(sel)[2]
+                    idx = tree.indexFromItem(item)
+                    item_selection.append(QtGui.QItemSelectionRange(idx))
+
+                tree.selectionModel().select(item_selection,
+                    QtGui.QItemSelectionModel.ClearAndSelect)
+            else:
+                tree.setCurrentItem(self._object_info(selection)[2])
         except:
-            pass
+            from traitsui.api import raise_to_debug
+            raise_to_debug()
 
     #---------------------------------------------------------------------------
     #  Handles the 'selected' trait being changed:
@@ -210,6 +242,11 @@ class SimpleEditor ( Editor ):
         """
         self._veto = True
 
+    def _refresh_changed ( self ):
+        """ Update the viewport.
+        """
+        self._tree.viewport().update()
+
     #---------------------------------------------------------------------------
     #  Disposes of the contents of an editor:
     #---------------------------------------------------------------------------
@@ -252,19 +289,28 @@ class SimpleEditor ( Editor ):
             editor.
         """
         tree = self._tree
+        if tree is None:
+            return
         saved_state = {}
 
+        object, node = self._node_for( self.old_value )
+        old_nid = self._get_object_nid( object, node.get_children_id(object))
+        if old_nid: self._delete_node(old_nid)
+
+        object, node = self._node_for( self.old_value )
+        old_nid = self._get_object_nid( object, node.get_children_id(object) )
+        if old_nid:
+            self._delete_node(old_nid)
+
         tree.clear()
+        self._map = {}
 
         object, node = self._node_for( self.value )
         if node is not None:
             if self.factory.hide_root:
                 nid = tree.invisibleRootItem()
             else:
-                nid = QtGui.QTreeWidgetItem(tree)
-                nid.setText(0, node.get_label(object))
-                nid.setIcon(0, self._get_icon(node, object))
-                nid.setToolTip(0, node.get_tooltip(object))
+                nid = self._create_item(tree, node, object)
 
             self._map[ id( object ) ] = [ ( node.get_children_id(object), nid ) ]
             self._add_listeners( node, object )
@@ -276,6 +322,10 @@ class SimpleEditor ( Editor ):
                     tree.setCurrentItem(nid)
 
             self.expand_levels( nid, self.factory.auto_open, False )
+        ncolumns = self._tree.columnCount()
+        if ncolumns > 1:
+            for i in range(ncolumns):
+                self._tree.resizeColumnToContents(i)
         # FIXME: Clear the current editor (if any)...
 
     #---------------------------------------------------------------------------
@@ -287,6 +337,101 @@ class SimpleEditor ( Editor ):
         """
         return self._tree
 
+    def _get_brush(self, color) :
+        if isinstance(color, SequenceTypes):
+            q_color = QtGui.QColor(*color)
+        else:
+            q_color = QtGui.QColor(color)
+        return QtGui.QBrush(q_color)
+
+
+    def _set_column_labels(self, nid, column_labels):
+        """ Set the column labels.
+        """
+        for i, (header, label) in enumerate(map(None,
+            self.factory.column_headers[1:], column_labels), 1):
+            if header is not None and label is not None:
+                nid.setText(i, label)
+
+    #---------------------------------------------------------------------------
+        color = node.get_background(object)
+        if color : nid.setBackground(0, self._get_brush(color))
+        color = node.get_foreground(object)
+        if color : nid.setForeground(0, self._get_brush(color))
+    #  Private Delegate class to do drawing in case of wrapped text labels
+    #---------------------------------------------------------------------------
+
+    class ItemDelegate(QtGui.QStyledItemDelegate):
+        """ A delegate class to draw wrapped text labels """
+        # FIXME: sizeHint() should return the size required by the label,
+        # which is dependent on the width available, which is different for
+        # each item due to the nested tree structure. However the option.rect
+        # argument available to the sizeHint() is invalid (width=-1) so as a
+        # hack sizeHintChanged is emitted in paint() and the size of drawn
+        # text is returned, as paint() gets a valid option.rect argument.
+
+        def __init__(self, *args, **kwargs):
+            self.size_map = collections.defaultdict(lambda:QtCore.QSize(1,21))
+            QtGui.QStyledItemDelegate.__init__(self, *args, **kwargs)
+
+        def sizeHint(self, option, index):
+            """ returns area taken by the text. """
+            return self.size_map[self.editor._tree.itemFromIndex(index)]
+
+        def paint(self, painter, option, index):
+            """ Do the actual drawing of the text """
+            # For icon and highlights during selection etc
+            super(self.__class__, self).paint(painter, option, index)
+
+            item = self.editor._tree.itemFromIndex(index)
+            expanded, node, object = self.editor._get_node_data(item)
+            text = node.get_label(object)
+            textrect = option.rect
+            if self.editor.factory.show_icons:
+                iconwidth = 24 # FIXME: get width from actual
+            else:
+                iconwidth = 0
+            rect = painter.drawText(option.rect.left() + iconwidth,
+                                    option.rect.top(),
+                                    option.rect.width() - iconwidth,
+                                    option.rect.height(),
+                                    QtCore.Qt.TextWordWrap, text)
+            # Need to set the appropriate sizeHint of the item.
+            if self.size_map[item] != rect.size():
+                self.size_map[item] = rect.size()
+                do_later(self.sizeHintChanged.emit, index)
+
+
+    #---------------------------------------------------------------------------
+    #  Create a TreeWidgetItem as per word wrap policy and set icon,tooltip
+    #---------------------------------------------------------------------------
+
+    def _create_item(self, nid, node, object, index=None):
+        """ Create  a new TreeWidgetItem as per word_wrap policy.
+
+        Index is the index of the new node in the parent:
+            None implies append the child to the end. """
+        if index is None:
+            cnid = QtGui.QTreeWidgetItem(nid)
+        else:
+            cnid = QtGui.QTreeWidgetItem()
+            nid.insertChild(index, cnid)
+        if self.factory.word_wrap:
+            item = self.ItemDelegate()
+            item.editor = self
+            self._tree.setItemDelegate(item)
+        else:
+            cnid.setText(0, node.get_label(object))
+        cnid.setIcon(0, self._get_icon(node, object))
+        cnid.setToolTip(0, node.get_tooltip(object))
+        return cnid
+
+    def _set_label(self, nid, text, col=0):
+        """ Set the label of the specified item """
+        if not self.factory.word_wrap or col!=0:
+            expanded, node, object = self._get_node_data(nid)
+            nid.setText(col, node.get_label(object))
+
     #---------------------------------------------------------------------------
     #  Appends a new node to the specified node:
     #---------------------------------------------------------------------------
@@ -294,10 +439,17 @@ class SimpleEditor ( Editor ):
     def _append_node ( self, nid, node, object ):
         """ Appends a new node to the specified node.
         """
-        cnid = QtGui.QTreeWidgetItem(nid)
-        cnid.setText(0, node.get_label(object))
-        cnid.setIcon(0, self._get_icon(node, object))
-        cnid.setToolTip(0, node.get_tooltip(object))
+        return self._insert_node( nid, None, node, object )
+
+    #---------------------------------------------------------------------------
+    #  Inserts a new node to the specified node:
+    #---------------------------------------------------------------------------
+
+    def _insert_node ( self, nid, index, node, object ):
+        """ Inserts a new node before a specified index into the children of the
+            specified node.
+        """
+        cnid = self._create_item(nid, node, object, index)
 
         has_children = self._has_children(node, object)
         self._set_node_data( cnid, ( False, node, object ) )
@@ -339,18 +491,23 @@ class SimpleEditor ( Editor ):
             del pnid._dummy
             return
 
-        expanded, node, object = self._get_node_data(nid)
-        id_object = id(object)
-        object_info = self._map[id_object]
-        for i, info in enumerate(object_info):
-            # QTreeWidgetItem does not have an equal operator, so use id()
-            if id(nid) == id(info[1]):
-                del object_info[i]
-                break
+        try:
+            expanded, node, object = self._get_node_data(nid)
+        except AttributeError:
+            # The node has already been deleted.
+            pass
+        else:
+            id_object = id(object)
+            object_info = self._map[id_object]
+            for i, info in enumerate(object_info):
+                # QTreeWidgetItem does not have an equal operator, so use id()
+                if id(nid) == id(info[1]):
+                    del object_info[i]
+                    break
 
-        if len( object_info ) == 0:
-            self._remove_listeners( node, object )
-            del self._map[ id_object ]
+            if len( object_info ) == 0:
+                self._remove_listeners( node, object )
+                del self._map[ id_object ]
 
         if pnid is None:
             self._tree.takeTopLevelItem(self._tree.indexOfTopLevelItem(nid))
@@ -468,6 +625,7 @@ class SimpleEditor ( Editor ):
             node.when_children_changed(  object, self._children_updated,  False)
 
         node.when_label_changed( object, self._label_updated, False )
+        node.when_column_labels_change(object, self._column_labels_updated, False)
 
     #---------------------------------------------------------------------------
     #  Removes any event listeners from a specified object:
@@ -481,6 +639,7 @@ class SimpleEditor ( Editor ):
             node.when_children_changed(  object, self._children_updated,  True )
 
         node.when_label_changed( object, self._label_updated, True )
+        node.when_column_labels_change(object, self._column_labels_updated, False)
 
     #---------------------------------------------------------------------------
     #  Returns the tree node data for a specified object in the form
@@ -833,6 +992,18 @@ class SimpleEditor ( Editor ):
         # Fire the 'dclick' event with the clicked on object as value:
         self.dclick = object
 
+    #---------------------------------------------------------------------------
+    #  Handles a tree item being activated:
+    #---------------------------------------------------------------------------
+
+    def _on_item_activated(self, nid, col):
+        """ Handles a tree item being activated.
+        """
+        _, node, object = self._get_node_data(nid)
+
+        # Fire the 'activated' event with the clicked on object as value:
+        self.activated = object
+
     #---------------------------------------------------------------------------
     #  Handles a tree node being selected:
     #---------------------------------------------------------------------------
@@ -889,8 +1060,9 @@ class SimpleEditor ( Editor ):
             if object is not None:
                 # Try to chain the undo history to the main undo history:
                 view = node.get_view( object )
-                if view is None:
-                    view = object.trait_view()
+                if view is None or isinstance(view, str) :
+                    view = object.trait_view(view)
+
                 if (self.ui.history is not None) or (view.kind == 'subpanel'):
                     ui = object.edit_traits( parent = editor,
                                              view   = view,
@@ -1073,6 +1245,23 @@ class SimpleEditor ( Editor ):
 
         return can_rename
 
+    def _is_droppable ( self, node, object, add_object, for_insert ):
+        """ Returns whether a given object is droppable on the node.
+        """
+        if for_insert and (not node.can_insert( object )):
+            return False
+
+        return node.can_add( object, add_object )
+
+    def _drop_object ( self, node, object, dropped_object, make_copy = True ):
+        """ Returns a droppable version of a specified object.
+        """
+        new_object = node.drop_object( object, dropped_object )
+        if (new_object is not dropped_object) or (not make_copy):
+            return new_object
+
+        return copy.deepcopy( new_object )
+
 #----- pyface.action 'controller' interface implementation: --------------------
 
     #---------------------------------------------------------------------------
@@ -1103,18 +1292,12 @@ class SimpleEditor ( Editor ):
         """ Returns whether the action should be defined in the user interface.
         """
         if action.defined_when != '':
-            try:
-                if not eval( action.defined_when, globals(), self._context ):
-                    return False
-            except:
-                open_fbi()
+            if not eval( action.defined_when, globals(), self._context ):
+                return False
 
         if action.visible_when != '':
-            try:
-                if not eval( action.visible_when, globals(), self._context ):
-                    return False
-            except:
-                open_fbi()
+            if not eval( action.visible_when, globals(), self._context ):
+                return False
 
         return True
 
@@ -1155,8 +1338,8 @@ class SimpleEditor ( Editor ):
                         'info':    info,
                         'handler': handler } )
             except:
-                # fixme: Should the exception be logged somewhere?
-                pass
+                from traitsui.api import raise_to_debug
+                raise_to_debug()
             return
 
         method = getattr( handler, method_name, None )
@@ -1184,8 +1367,9 @@ class SimpleEditor ( Editor ):
             try:
                 if not eval( condition, globals(), self._context ):
                     value = False
-            except:
-                open_fbi()
+            except Exception as e:
+                logger.warning("Exception (%s) raised when evaluating the "
+                               "condition %s. Returning True." % (e,condition))
             setattr( object, trait, value )
 
 #----- Menu event handlers: ----------------------------------------------------
@@ -1276,7 +1460,7 @@ class SimpleEditor ( Editor ):
             if new_label != '':
                 node.set_label(object, new_label)
             else:
-                nid.setText(col, old_label)
+                self._set_label(nid, old_label, col)
 
     #---------------------------------------------------------------------------
     #  Adds a new object to the current node:
@@ -1342,15 +1526,15 @@ class SimpleEditor ( Editor ):
         start = event.index
         n     = len( event.added )
         end   = start + len( event.removed )
-        tree                = self._tree
+        tree  = self._tree
 
         for expanded, node, nid in self._object_info_for( object, name ):
             children = node.get_children( object )
 
             # If the new children aren't all at the end, remove/add them all:
-            if (n > 0) and ((start + n) != len( children )):
-                self._children_replaced( object, name, event )
-                return
+            #if (n > 0) and ((start + n) != len( children )):
+            #    self._children_replaced( object, name, event )
+            #    return
 
             # Only add/remove the changes if the node has already been expanded:
             if expanded:
@@ -1358,11 +1542,17 @@ class SimpleEditor ( Editor ):
                 for cnid in self._nodes_for( nid )[ start: end ]:
                     self._delete_node( cnid )
 
+                remaining = n - len( event.removed )
+                child_index = 0
                 # Add all of the children that were added:
                 for child in event.added:
                     child, child_node = self._node_for( child )
                     if child_node is not None:
-                        self._append_node( nid, child_node, child )
+                        insert_index = (start + child_index) if \
+                                        (start <= remaining) else None
+                        self._insert_node( nid, insert_index, child_node,
+                                        child )
+                        child_index += 1
 
             # Try to expand the node (if requested):
             if node.can_auto_open( object ):
@@ -1383,11 +1573,28 @@ class SimpleEditor ( Editor ):
             if nid not in nids:
                 nids[ nid ] = None
                 node = self._get_node_data( nid )[1]
-                nid.setText(0, node.get_label(object))
+                self._set_label(nid, node.get_label(object), 0)
                 self._update_icon(nid)
 
         self._tree.blockSignals(blk)
 
+    def _column_labels_updated(self, object, name, new):
+        """  Handles the column labels of an object being changed.
+        """
+        # Prevent the itemChanged() signal from being emitted.
+        blk = self._tree.blockSignals(True)
+
+        nids = {}
+        for name2, nid in self._map[ id( object ) ]:
+            if nid not in nids:
+                nids[ nid ] = None
+                node = self._get_node_data( nid )[1]
+                # Just do all of them at once. The number of columns should be
+                # small.
+                self._set_column_labels(nid, node.get_column_labels(object))
+
+        self._tree.blockSignals(blk)
+
 #-- UI preference save/restore interface ---------------------------------------
 
     #---------------------------------------------------------------------------
@@ -1406,6 +1613,11 @@ class SimpleEditor ( Editor ):
                 structure = prefs
 
             self.control.restoreState(structure)
+        header = self._tree.header()
+        self.setExpandsOnDoubleClick(editor.factory.expands_on_dclick)
+
+        if header is not None and 'column_state' in prefs:
+            header.restoreState(prefs['column_state'])
 
     #---------------------------------------------------------------------------
     #  Returns any user preference information associated with the editor:
@@ -1414,10 +1626,14 @@ class SimpleEditor ( Editor ):
     def save_prefs ( self ):
         """ Returns any user preference information associated with the editor.
         """
+        prefs = {}
         if isinstance(self.control, QtGui.QSplitter):
-            return {'structure': str(self.control.saveState())}
+            prefs['structure'] = str(self.control.saveState())
+        header = self._tree.header()
+        if header is not None:
+            prefs['column_state'] = str(header.saveState())
 
-        return None
+        return prefs
 
 #-- End UI preference save/restore interface -----------------------------------
 
@@ -1435,35 +1651,52 @@ class _TreeWidget(QtGui.QTreeWidget):
         """
         QtGui.QTreeWidget.__init__(self, parent)
 
-        self.header().hide()
         self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
+        # Set up headers if necessary.
+        column_count = len(editor.factory.column_headers)
+        if column_count > 0:
+            self.setHeaderHidden(False)
+            self.setColumnCount(column_count)
+            self.setHeaderLabels(editor.factory.column_headers)
+        else:
+            self.setHeaderHidden(True)
+
+        self.setAlternatingRowColors(editor.factory.alternating_row_colors)
+        padding = editor.factory.vertical_padding
+        if padding > 0:
+            self.setStyleSheet("""
+            QTreeView::item {
+                padding-top: %spx;
+                padding-bottom: %spx;
+            }
+            """ % (padding, padding))
 
         if editor.factory.selection_mode == 'extended':
             self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
 
-        self.connect(self, QtCore.SIGNAL('itemExpanded(QTreeWidgetItem *)'),
-                editor._on_item_expanded)
-        self.connect(self, QtCore.SIGNAL('itemCollapsed(QTreeWidgetItem *)'),
-                editor._on_item_collapsed)
-        self.connect(self,
-                QtCore.SIGNAL('itemClicked(QTreeWidgetItem *, int)'),
-                editor._on_item_clicked)
-        self.connect(self,
-                QtCore.SIGNAL('itemDoubleClicked(QTreeWidgetItem *, int)'),
-                editor._on_item_dclicked)
-        self.connect(self, QtCore.SIGNAL('itemSelectionChanged()'),
-                editor._on_tree_sel_changed)
-        self.connect(self, QtCore.SIGNAL('customContextMenuRequested(QPoint)'),
-                editor._on_context_menu)
-        self.connect(self,
-                QtCore.SIGNAL('itemChanged(QTreeWidgetItem *, int)'),
-                editor._on_nid_changed)
+        self.itemExpanded.connect(editor._on_item_expanded)
+        self.itemCollapsed.connect(editor._on_item_collapsed)
+        self.itemClicked.connect(editor._on_item_clicked)
+        self.itemDoubleClicked.connect(editor._on_item_dclicked)
+        self.itemActivated.connect(editor._on_item_activated)
+        self.itemSelectionChanged.connect(editor._on_tree_sel_changed)
+        self.customContextMenuRequested.connect(editor._on_context_menu)
+        self.itemChanged.connect(editor._on_nid_changed)
 
         self._editor = editor
         self._dragging = None
 
+    def resizeEvent(self, event):
+        """ Overridden to emit sizeHintChanged() of items for word wrapping """
+        if self._editor.factory.word_wrap:
+            for i in range(self.topLevelItemCount()):
+                mi = self.indexFromItem(self.topLevelItem(i))
+                id = self.itemDelegate(mi)
+                id.sizeHintChanged.emit(mi)
+        super(self.__class__, self).resizeEvent(event)
+
     def startDrag(self, actions):
         """ Reimplemented to start the drag of a tree widget item.
         """
@@ -1557,7 +1790,7 @@ class _TreeWidget(QtGui.QTreeWidget):
         # See if the model will accept a drop.
         data = PyMimeData.coerce(e.mimeData()).instance()
 
-        if not node._is_droppable(object, data, insert):
+        if not self._editor._is_droppable(node, object, data, insert):
             return
 
         e.acceptProposedAction()
@@ -1583,11 +1816,11 @@ class _TreeWidget(QtGui.QTreeWidget):
         _, node, object = editor._get_node_data(nid)
 
         if e.proposedAction() == QtCore.Qt.MoveAction:
-            if not node._is_droppable( object, data, False ):
+            if not self._editor._is_droppable(node, object, data, False ):
                 return
 
             if dragging is not None:
-                data = node._drop_object( object, data, False )
+                data = self._editor._drop_object( node, object, data, False )
                 if data is not None:
                     try:
                         editor._begin_undo()
@@ -1597,14 +1830,14 @@ class _TreeWidget(QtGui.QTreeWidget):
                     finally:
                         editor._end_undo()
             else:
-                data = node._drop_object( object, data )
+                data = self._editor._drop_object( node, object, data, True )
                 if data is not None:
                     editor._undoable_append( node, object, data, False )
         else:
             to_node, to_object, to_index = editor._node_index( nid )
             if to_node is not None:
                 if dragging is not None:
-                    data = node._drop_object( to_object, data, False )
+                    data = self._editor._drop_object( node, to_object, data, False )
                     if data is not None:
                         from_node, from_object, from_index = \
                             editor._node_index( dragging )
@@ -1620,7 +1853,7 @@ class _TreeWidget(QtGui.QTreeWidget):
                         finally:
                             editor._end_undo()
                 else:
-                    data = to_node._drop_object( to_object, data )
+                    data = self._editor._drop_object( to_node, to_object, data, True )
                     if data is not None:
                         editor._undoable_insert( to_node, to_object, to_index,
                                                data, False )
