@@ -25,12 +25,12 @@
 
 from __future__ import absolute_import
 
-from traits.api import (AdaptedTo, Adapter, Any, Bool, Callable, HasPrivateTraits,
-    Instance, Interface, List, Property, Str, cached_property)
+from traits.api import (AdaptedTo, Adapter, Any, Bool, Callable, Either,
+    HasPrivateTraits, Instance, Interface, List, Property, Str, cached_property)
 
 from traits.trait_base import SequenceTypes, get_resource_path, xgetattr, xsetattr
 
-from .view import View
+from .ui_traits import AView
 
 #-------------------------------------------------------------------------------
 #  'TreeNode' class:
@@ -52,6 +52,9 @@ class TreeNode ( HasPrivateTraits ):
     # the string starts with '='.
     label = Str
 
+    # The name of a trait containing a list of labels for any columns.
+    column_labels = Str
+
     # Either the name of a trait containing a tooltip, or constant tooltip, if
     # the string starts with '='.
     tooltip = Str
@@ -101,6 +104,9 @@ class TreeNode ( HasPrivateTraits ):
     # Function for formatting the label
     formatter = Callable
 
+    # Functions for formatting the other columns.
+    column_formatters = List(Either(None, Callable))
+
     # Function for formatting the tooltip
     tooltip_formatter = Callable
 
@@ -114,7 +120,7 @@ class TreeNode ( HasPrivateTraits ):
     on_dclick = Callable
 
     # View to use for editing the object
-    view = Instance( View )
+    view = AView
 
     # Right-click context menu. The value can be one of:
     #
@@ -135,6 +141,12 @@ class TreeNode ( HasPrivateTraits ):
     # Resource path used to locate the node icon
     icon_path = Str
 
+    # Selector or name for background color
+    background = Any
+
+    # Selector or name for foreground color
+    foreground = Any
+
     # fixme: The 'menu' trait should really be defined as:
     #        Instance( 'traitsui.menu.MenuBar' ), but it doesn't work
     #        right currently.
@@ -310,6 +322,37 @@ class TreeNode ( HasPrivateTraits ):
             object.on_trait_change( listener, label, remove = remove,
                                     dispatch = 'ui' )
 
+    def get_column_labels(self, object):
+        """ Get the labels for any columns that have been defined.
+        """
+        trait = self.column_labels
+        labels = xgetattr(object, trait, [])
+        formatted = []
+        for formatter, label in map(None, self.column_formatters, labels):
+            # If the list of column formatters is shorter than the list of
+            # labels, then map(None) will extend it with Nones. Just pass the label
+            # as preformatted. Similarly, explicitly using None in the list will
+            # pass through the item.
+            if formatter is None:
+                formatted.append(label)
+            else:
+                formatted.append(formatter(label))
+        return formatted
+
+    def when_column_labels_change(self, object, listener, remove):
+        """ Sets up or removes a listener for the column labels being changed on
+        a specified object.
+
+        This will fire when either the list is reassigned or when it is
+        modified. I.e., it listens both to the trait change event and the
+        trait_items change event. Implement the listener appropriately to handle
+        either case.
+        """
+        trait = self.column_labels
+        if trait != '':
+            object.on_trait_change(listener, trait, remove=remove, dispatch='ui')
+            object.on_trait_change(listener, trait+'_items', remove=remove, dispatch='ui')
+
     #---------------------------------------------------------------------------
     #  Gets the tooltip to display for a specified object:
     #---------------------------------------------------------------------------
@@ -384,6 +427,18 @@ class TreeNode ( HasPrivateTraits ):
         """
         return self.menu
 
+    def get_background(self, object) :
+        background = self.background
+        if isinstance(background, basestring) :
+            background = getattr(object, background, background)
+        return background
+
+    def get_foreground(self, object) :
+        foreground = self.foreground
+        if isinstance(foreground, basestring) :
+            foreground = getattr(object, foreground, foreground)
+        return foreground
+
     #---------------------------------------------------------------------------
     #  Returns whether or not the object's children can be renamed:
     #---------------------------------------------------------------------------
@@ -663,6 +718,20 @@ class ITreeNode ( Interface ):
             specified object.
         """
 
+    def get_column_labels(self, object):
+        """ Get the labels for any columns that have been defined.
+        """
+
+    def when_column_labels_change(self, object, listener, remove):
+        """ Sets up or removes a listener for the column labels being changed on
+        a specified object.
+
+        This will fire when either the list is reassigned or when it is
+        modified. I.e., it listens both to the trait change event and the
+        trait_items change event. Implement the listener appropriately to handle
+        either case.
+        """
+
     def get_tooltip ( self ):
         """ Gets the tooltip to display for a specified object.
         """
@@ -850,6 +919,22 @@ class ITreeNodeAdapter ( Adapter ):
         """
         pass
 
+    def get_column_labels(self):
+        """ Get the labels for any columns that have been defined.
+        """
+        return []
+
+    def when_column_labels_change(self, listener, remove):
+        """ Sets up or removes a listener for the column labels being changed on
+        a specified object.
+
+        This will fire when either the list is reassigned or when it is
+        modified. I.e., it listens both to the trait change event and the
+        trait_items change event. Implement the listener appropriately to handle
+        either case.
+        """
+        pass
+
     def get_tooltip ( self ):
         """ Gets the tooltip to display for a specified object.
         """
@@ -881,6 +966,16 @@ class ITreeNodeAdapter ( Adapter ):
         """
         return None
 
+    def get_background ( self ):
+        """ Returns the background for object
+        """
+        return None
+
+    def get_foreground ( self ):
+        """ Returns the foreground for object
+        """
+        return None
+
     def can_rename ( self ):
         """ Returns whether the object's children can be renamed.
         """
@@ -1049,6 +1144,22 @@ class ITreeNodeAdapterBridge ( HasPrivateTraits ):
         """
         return self.adapter.when_label_changed( listener, remove )
 
+    def get_column_labels(self, object):
+        """ Get the labels for any columns that have been defined.
+        """
+        return self.adapter.get_column_labels()
+
+    def when_column_labels_change(self, object, listener, remove):
+        """ Sets up or removes a listener for the column labels being changed on
+        a specified object.
+
+        This will fire when either the list is reassigned or when it is
+        modified. I.e., it listens both to the trait change event and the
+        trait_items change event. Implement the listener appropriately to handle
+        either case.
+        """
+        return self.adapter.when_column_labels_change(listener, remove)
+
     def get_tooltip ( self, object ):
         """ Gets the tooltip to display for a specified object.
         """
@@ -1080,6 +1191,16 @@ class ITreeNodeAdapterBridge ( HasPrivateTraits ):
         """
         return self.adapter.get_menu()
 
+    def get_background ( self, object ):
+        """ Returns the background for object
+        """
+        return self.adapter.get_background()
+
+    def get_foreground ( self, object ):
+        """ Returns the foreground for object
+        """
+        return self.adapter.get_foreground()
+
     def can_rename ( self, object ):
         """ Returns whether the object's children can be renamed.
         """
@@ -1158,6 +1279,10 @@ class ITreeNodeAdapterBridge ( HasPrivateTraits ):
         """
         return self.adapter.dclick()
 
+
+# FIXME RTK: add the column_labels API to the following TreeNodes, too.
+
+
 #-------------------------------------------------------------------------------
 #  'ObjectTreeNode' class
 #-------------------------------------------------------------------------------
